@@ -559,6 +559,9 @@ confront_table_data <- function(
    stopifnot(is.RelTableModel(x))
    ## Matrix model ----
    if(is.MatrixModel(x)){
+      if(!inherits(d, c("matrix", "Matrix"))){
+         stop(x$tableName, " is neither a matrix nor a Matrix")
+      }
       vf <- x$fields %>% dplyr::filter(!.data$type %in% c("row", "column"))
       toRet <- list(
          missingFields = character(0),
@@ -606,9 +609,9 @@ confront_table_data <- function(
       return(toRet)
    }
 
-   stopifnot(
-      is.data.frame(d)
-   )
+   if(!inherits(d, "data.frame")){
+      stop(x$tableName, " is not a data.frame")
+   }
    ## Available fields ----
    missingFields <- setdiff(x$fields$name, colnames(d))
    suppFields <- setdiff(colnames(d), x$fields$name)
@@ -656,7 +659,9 @@ confront_table_data <- function(
             }
          }
       }
-      if("unique" %in% checks && fu && any(duplicated(dplyr::pull(d, !!fn)))){
+      toTest <- dplyr::pull(d, !!fn)
+      toTest <- toTest[which(!is.na(toTest))]
+      if("unique" %in% checks && fu && any(duplicated(toTest))){
          toRet$fields[[fn]]$success <- FALSE
          toRet$success <- FALSE
          toRet$fields[[fn]]$message <- paste(c(
@@ -682,7 +687,11 @@ confront_table_data <- function(
             if(!idx$unique){
                toRet$indexes[[i]]$success <- TRUE
             }else{
-               if(any(duplicated(d[,idx$fields]))){
+               toTest <- dplyr::filter(
+                  d[,idx$fields],
+                  apply(d[,idx$fields], 1, function(x) all(!is.na(x)))
+               )
+               if(any(duplicated(toTest))){
                   toRet$indexes[[i]]$success <- FALSE
                   toRet$indexes[[i]]$message <- paste(c(
                      toRet$indexes[[i]]$message,
